@@ -24,6 +24,13 @@ globalVariables(c('.'
                   ,'xend'
                   ,'y'
                   ,'yend'
+                  , 'CLASS_NAME'
+                  , 'MATCH'
+                  , 'METHOD_NAME'
+                  , 'METHOD_TYPE'
+                  , 'PARENT_IN_PKG'
+                  , 'PARENT_NAME'
+                  , 'SYMBOL'
                   ))
 
 # NULL object for common parameter documentation
@@ -35,4 +42,4 @@ globalVariables(c('.'
 #' @title NULL Object For Common Documentation
 #' @description This is a NULL object with documentation so that later functions can call
 #' inheritParams
-NULL
\ No newline at end of file
+NULL
